@@ -41,22 +41,15 @@
 
 #include <glib-object.h>
 #include <glib/gi18n.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkiconfactory.h>
+#include <gtk/gtk.h>
 
 #include "gedit-notebook.h"
 #include "gedit-marshal.h"
 #include "gedit-window.h"
-#include "gedit-tooltips.h"
+
+#ifdef BUILD_SPINNER
 #include "gedit-spinner.h"
+#endif
 
 #define AFTER_ALL_TABS -1
 #define NOT_IN_APP_WINDOWS -2
@@ -66,7 +59,6 @@
 struct _GeditNotebookPrivate
 {
 	GList         *focused_pages;
-	GeditTooltips *title_tips;
 	gulong         motion_notify_handler_id;
 	gint           x_start;
 	gint           y_start;
@@ -80,6 +72,9 @@ G_DEFINE_TYPE(GeditNotebook, gedit_notebook, GTK_TYPE_NOTEBOOK)
 
 static void gedit_notebook_finalize (GObject *object);
 
+static gboolean gedit_notebook_change_current_page (GtkNotebook *notebook,
+						    gint         offset);
+
 static void move_current_tab_to_another_notebook  (GeditNotebook  *src,
 						   GeditNotebook  *dest,
 						   GdkEventMotion *event,
@@ -105,8 +100,11 @@ static void
 gedit_notebook_class_init (GeditNotebookClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkNotebookClass *notebook_class = GTK_NOTEBOOK_CLASS (klass);
 
 	object_class->finalize = gedit_notebook_finalize;
+	
+	notebook_class->change_current_page = gedit_notebook_change_current_page;
 
 	signals[TAB_ADDED] =
 		g_signal_new ("tab_added",
@@ -159,6 +157,17 @@ gedit_notebook_class_init (GeditNotebookClass *klass)
 			      GEDIT_TYPE_TAB);
 
 	g_type_class_add_private (object_class, sizeof(GeditNotebookPrivate));
+	
+	/* Set up a style for the close button with no focus padding. */
+	gtk_rc_parse_string (
+		"style \"gedit-close-button-style\"\n"
+		"{\n"
+		"  GtkWidget::focus-padding = 0\n"
+		"  GtkWidget::focus-line-width = 0\n"
+		"  xthickness = 0\n"
+		"  ythickness = 0\n"
+		"}\n"
+		"widget \"*.gedit-close-button\" style \"gedit-close-button-style\"");
 }
 
 static GeditNotebook *
@@ -296,9 +305,18 @@ find_notebook_and_tab_at_pos (gint            abs_x,
 	}
 }
 
-/* If dest_position is greater than or equal to the number of tabs 
-   of the destination nootebook or negative, tab will be moved to the 
-   end of the tabs. */
+/**
+ * gedit_notebook_move_tab:
+ * @src: a #GeditNotebook
+ * @dest: a #GeditNotebook
+ * @tab: a #GeditTab
+ * @dest_position: the position for @tab
+ *
+ * Moves @tab from @src to @dest.
+ * If dest_position is greater than or equal to the number of tabs 
+ * of the destination nootebook or negative, tab will be moved to the 
+ * end of the tabs.
+ */
 void
 gedit_notebook_move_tab (GeditNotebook *src,
 			 GeditNotebook *dest,
@@ -317,9 +335,17 @@ gedit_notebook_move_tab (GeditNotebook *src,
 	g_object_unref (tab);
 }
 
-/* If dest_position is greater than or equal to the number of tabs 
-   of the destination nootebook or negative, tab will be moved to the 
-   end of the tabs. */
+/**
+ * gedit_notebook_reorder_tab:
+ * @src: a #GeditNotebook
+ * @tab: a #GeditTab
+ * @dest_position: the position for @tab
+ *
+ * Reorders the page containing @tab, so that it appears in @dest_position position.
+ * If dest_position is greater than or equal to the number of tabs 
+ * of the destination notebook or negative, tab will be moved to the 
+ * end of the tabs.
+ */
 void
 gedit_notebook_reorder_tab (GeditNotebook *src,
 			    GeditTab      *tab,
@@ -365,7 +391,7 @@ drag_start (GeditNotebook *notebook,
 	/* FIXME multi-head */
 	if (!gdk_pointer_is_grabbed ())
 	{
-		gdk_pointer_grab (GTK_WIDGET (notebook)->window,
+		gdk_pointer_grab (gtk_widget_get_window (GTK_WIDGET (notebook)),
 				  FALSE,
 				  GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
 				  NULL, 
@@ -599,6 +625,13 @@ button_press_cb (GeditNotebook  *notebook,
 	return FALSE;
 }
 
+/**
+ * gedit_notebook_new:
+ *
+ * Creates a new #GeditNotebook object.
+ *
+ * Returns: a new #GeditNotebook
+ */
 GtkWidget *
 gedit_notebook_new (void)
 {
@@ -665,9 +698,6 @@ gedit_notebook_init (GeditNotebook *notebook)
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
 
-	notebook->priv->title_tips = gedit_tooltips_new ();
-	g_object_ref_sink (notebook->priv->title_tips);
-
 	notebook->priv->always_show_tabs = TRUE;
 
 	g_signal_connect (notebook, 
@@ -694,12 +724,53 @@ gedit_notebook_finalize (GObject *object)
 
 	if (notebook->priv->focused_pages)
 		g_list_free (notebook->priv->focused_pages);
-		
-	g_object_unref (notebook->priv->title_tips);
 
 	G_OBJECT_CLASS (gedit_notebook_parent_class)->finalize (object);
 }
 
+/*
+ * We need to override this because when we don't show the tabs, like in
+ * fullscreen we need to have wrap around too
+ */
+static gboolean
+gedit_notebook_change_current_page (GtkNotebook *notebook,
+				    gint         offset)
+{
+	gboolean wrap_around;
+	gint current;
+	
+	current = gtk_notebook_get_current_page (notebook);
+
+	if (current != -1)
+	{
+		current = current + offset;
+		
+		g_object_get (gtk_widget_get_settings (GTK_WIDGET (notebook)),
+			      "gtk-keynav-wrap-around", &wrap_around,
+			      NULL);
+		
+		if (wrap_around)
+		{
+			if (current < 0)
+			{
+				current = gtk_notebook_get_n_pages (notebook) - 1;
+			}
+			else if (current >= gtk_notebook_get_n_pages (notebook))
+			{
+				current = 0;
+			}
+		}
+		
+		gtk_notebook_set_current_page (notebook, current);
+	}
+	else
+	{
+		gtk_widget_error_bell (GTK_WIDGET (notebook));
+	}
+
+	return TRUE;
+}
+
 static void
 sync_name (GeditTab *tab, GParamSpec *pspec, GtkWidget *hbox)
 {
@@ -708,12 +779,10 @@ sync_name (GeditTab *tab, GParamSpec *pspec, GtkWidget *hbox)
 	GtkWidget *ebox;
 	GtkWidget *button;
 	GtkWidget *spinner;
-	GeditTooltips *tips;	
 	gchar *str;
 	GtkImage *icon;
 	GeditTabState  state;
 	
-	tips = GEDIT_TOOLTIPS (g_object_get_data (G_OBJECT (hbox), "tooltips"));
 	label = GTK_WIDGET (g_object_get_data (G_OBJECT (hbox), "label"));
 	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (hbox), "label-ebox"));
 	icon = GTK_IMAGE (g_object_get_data (G_OBJECT (hbox), "icon"));
@@ -722,9 +791,8 @@ sync_name (GeditTab *tab, GParamSpec *pspec, GtkWidget *hbox)
 
 	nb = GEDIT_NOTEBOOK (gtk_widget_get_parent (GTK_WIDGET (tab)));
 
-	g_return_if_fail ((tips    != NULL) && 
-			  (label   != NULL) && 
-			  (ebox    != NULL) && 
+	g_return_if_fail ((label   != NULL) &&
+			  (ebox    != NULL) &&
 			  (button  != NULL) &&
 			  (icon    != NULL) &&
 			  (spinner != NULL) &&
@@ -739,7 +807,7 @@ sync_name (GeditTab *tab, GParamSpec *pspec, GtkWidget *hbox)
 	str = _gedit_tab_get_tooltips (tab);
 	g_return_if_fail (str != NULL);
 	
-	gedit_tooltips_set_tip (tips, ebox, str, NULL);
+	gtk_widget_set_tooltip_markup (ebox, str);
 	g_free (str);
 		
 	state = gedit_tab_get_state (tab);
@@ -758,7 +826,11 @@ sync_name (GeditTab *tab, GParamSpec *pspec, GtkWidget *hbox)
 		gtk_widget_hide (GTK_WIDGET (icon));
 		
 		gtk_widget_show (spinner);
+#ifdef BUILD_SPINNER
 		gedit_spinner_start (GEDIT_SPINNER (spinner));
+#else
+		gtk_spinner_start (GTK_SPINNER (spinner));
+#endif
 	}
 	else
 	{
@@ -773,7 +845,11 @@ sync_name (GeditTab *tab, GParamSpec *pspec, GtkWidget *hbox)
 		gtk_widget_show (GTK_WIDGET (icon));
 		
 		gtk_widget_hide (spinner);
+#ifdef BUILD_SPINNER
 		gedit_spinner_stop (GEDIT_SPINNER (spinner));
+#else
+		gtk_spinner_stop (GTK_SPINNER (spinner));
+#endif
 	}
 }
 
@@ -788,17 +864,15 @@ close_button_clicked_cb (GtkWidget *widget,
 }
 
 static void
-tab_label_style_set_cb (GtkWidget *hbox,
-			GtkStyle *previous_style,
-			gpointer user_data)
+close_button_style_set_cb (GtkWidget *button,
+			   GtkStyle *previous_style,
+			   gpointer user_data)
 {
-	GtkWidget *button;
 	gint h, w;
 
-	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (hbox),
+	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (button),
 					   GTK_ICON_SIZE_MENU, &w, &h);
 
-	button = g_object_get_data (G_OBJECT (hbox), "close-button");
 	gtk_widget_set_size_request (button, w + 2, h + 2);
 }
 
@@ -809,7 +883,6 @@ build_tab_label (GeditNotebook *nb,
 	GtkWidget *hbox, *label_hbox, *label_ebox;
 	GtkWidget *label, *dummy_label;
 	GtkWidget *close_button;
-	GtkRcStyle *rcstyle;
 	GtkWidget *image;
 	GtkWidget *spinner;
 	GtkWidget *icon;
@@ -831,18 +904,14 @@ build_tab_label (GeditNotebook *nb,
 	gtk_button_set_focus_on_click (GTK_BUTTON (close_button), FALSE);
 
 	/* make it as small as possible */
-	rcstyle = gtk_rc_style_new ();
-	rcstyle->xthickness = rcstyle->ythickness = 0;
-	gtk_widget_modify_style (close_button, rcstyle);
-	gtk_rc_style_unref (rcstyle),
+	gtk_widget_set_name (close_button, "gedit-close-button");
 
 	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE,
 					  GTK_ICON_SIZE_MENU);
 	gtk_container_add (GTK_CONTAINER (close_button), image);
 	gtk_box_pack_start (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
 
-	gedit_tooltips_set_tip (nb->priv->title_tips, close_button,
-			      _("Close document"), NULL);
+	gtk_widget_set_tooltip_text (close_button, _("Close document"));
 
 	g_signal_connect (close_button,
 			  "clicked",
@@ -850,8 +919,12 @@ build_tab_label (GeditNotebook *nb,
 			  tab);
 
 	/* setup spinner */
+#ifdef BUILD_SPINNER
 	spinner = gedit_spinner_new ();
 	gedit_spinner_set_size (GEDIT_SPINNER (spinner), GTK_ICON_SIZE_MENU);
+#else
+	spinner = gtk_spinner_new ();
+#endif
 	gtk_box_pack_start (GTK_BOX (label_hbox), spinner, FALSE, FALSE, 0);
 
 	/* setup site icon, empty by default */
@@ -868,8 +941,8 @@ build_tab_label (GeditNotebook *nb,
 	gtk_box_pack_start (GTK_BOX (label_hbox), dummy_label, TRUE, TRUE, 0);
 	
 	/* Set minimal size */
-	g_signal_connect (hbox, "style-set",
-			  G_CALLBACK (tab_label_style_set_cb), NULL);
+	g_signal_connect (close_button, "style-set",
+			  G_CALLBACK (close_button_style_set_cb), NULL);
 	
 	gtk_widget_show (hbox);
 	gtk_widget_show (label_ebox);
@@ -886,11 +959,17 @@ build_tab_label (GeditNotebook *nb,
 	g_object_set_data (G_OBJECT (hbox), "icon", icon);
 	g_object_set_data (G_OBJECT (hbox), "close-button", close_button);
 	g_object_set_data (G_OBJECT (tab), "close-button", close_button);
-	g_object_set_data (G_OBJECT (hbox), "tooltips", nb->priv->title_tips);
 
 	return hbox;
 }
 
+/**
+ * gedit_notebook_set_always_show_tabs:
+ * @nb: a #GeditNotebook
+ * @show_tabs: %TRUE to always show the tabs
+ *
+ * Sets the visibility of the tabs in the @nb.
+ */
 void
 gedit_notebook_set_always_show_tabs (GeditNotebook *nb, 
 				     gboolean       show_tabs)
@@ -902,6 +981,15 @@ gedit_notebook_set_always_show_tabs (GeditNotebook *nb,
 	update_tabs_visibility (nb, FALSE);
 }
 
+/**
+ * gedit_notebook_add_tab:
+ * @nb: a #GeditNotebook
+ * @tab: a #GeditTab
+ * @position: the position where the @tab should be added
+ * @jump_to: %TRUE to set the @tab as active
+ *
+ * Adds the specified @tab to the @nb.
+ */
 void
 gedit_notebook_add_tab (GeditNotebook *nb,
 		        GeditTab      *tab,
@@ -990,22 +1078,17 @@ remove_tab (GeditTab      *tab,
 {
 	GtkWidget *label, *ebox;
 	gint position;
-	
+
 	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
-	
+
 	label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
 	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "label-ebox"));
-	gedit_tooltips_set_tip (GEDIT_TOOLTIPS (nb->priv->title_tips), 
-			        ebox, 
-			        NULL, 
-			        NULL);
-					      
+
 	g_signal_handlers_disconnect_by_func (tab,
 					      G_CALLBACK (sync_name), 
 					      label);
 
-	/**
-	 * we ref the tab so that it's still alive while the tabs_removed
+	/* we ref the tab so that it's still alive while the tabs_removed
 	 * signal is processed.
 	 */
 	g_object_ref (tab);
@@ -1019,6 +1102,13 @@ remove_tab (GeditTab      *tab,
 	g_object_unref (tab);	
 }
 
+/**
+ * gedit_notebook_remove_tab:
+ * @nb: a #GeditNotebook
+ * @tab: a #GeditTab
+ *
+ * Removes @tab from @nb.
+ */
 void
 gedit_notebook_remove_tab (GeditNotebook *nb,
 			   GeditTab      *tab)
@@ -1043,6 +1133,12 @@ gedit_notebook_remove_tab (GeditNotebook *nb,
 	remove_tab (tab, nb);
 }
 
+/**
+ * gedit_notebook_remove_all_tabs:
+ * @nb: a #GeditNotebook
+ *
+ * Removes all #GeditTab from @nb.
+ */
 void
 gedit_notebook_remove_all_tabs (GeditNotebook *nb)
 {	
@@ -1074,9 +1170,18 @@ set_close_buttons_sensitivity (GeditTab      *tab,
 				  (state != GEDIT_TAB_STATE_CLOSING) &&
 				  (state != GEDIT_TAB_STATE_SAVING)  &&
 				  (state != GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW) &&
+				  (state != GEDIT_TAB_STATE_PRINTING) &&
+				  (state != GEDIT_TAB_STATE_PRINT_PREVIEWING) &&				  
 				  (state != GEDIT_TAB_STATE_SAVING_ERROR));
 }
 
+/**
+ * gedit_notebook_set_close_buttons_sensitive:
+ * @nb: a #GeditNotebook
+ * @sensitive: %TRUE to make the buttons sensitive
+ *
+ * Sets whether the close buttons in the tabs of @nb are sensitive.
+ */
 void
 gedit_notebook_set_close_buttons_sensitive (GeditNotebook *nb,
 					    gboolean       sensitive)
@@ -1095,6 +1200,14 @@ gedit_notebook_set_close_buttons_sensitive (GeditNotebook *nb,
 			       nb);
 }
 
+/**
+ * gedit_notebook_get_close_buttons_sensitive:
+ * @nb: a #GeditNotebook
+ *
+ * Whether the close buttons are sensitive.
+ *
+ * Returns: %TRUE if the close buttons are sensitive
+ */
 gboolean
 gedit_notebook_get_close_buttons_sensitive (GeditNotebook *nb)
 {
@@ -1103,6 +1216,13 @@ gedit_notebook_get_close_buttons_sensitive (GeditNotebook *nb)
 	return nb->priv->close_buttons_sensitive;
 }
 
+/**
+ * gedit_notebook_set_tab_drag_and_drop_enabled:
+ * @nb: a #GeditNotebook
+ * @enable: %TRUE to enable the drag and drop
+ *
+ * Sets whether drag and drop of tabs in the @nb is enabled.
+ */
 void
 gedit_notebook_set_tab_drag_and_drop_enabled (GeditNotebook *nb,
 					      gboolean       enable)
@@ -1117,6 +1237,14 @@ gedit_notebook_set_tab_drag_and_drop_enabled (GeditNotebook *nb,
 	nb->priv->tab_drag_and_drop_enabled = enable;		
 }
 
+/**
+ * gedit_notebook_get_tab_drag_and_drop_enabled:
+ * @nb: a #GeditNotebook
+ *
+ * Whether the drag and drop is enabled in the @nb.
+ *
+ * Returns: %TRUE if the drag and drop is enabled.
+ */
 gboolean	
 gedit_notebook_get_tab_drag_and_drop_enabled (GeditNotebook *nb)
 {
