@@ -35,7 +35,6 @@
 
 #include <glib/gi18n.h>
 #include <gconf/gconf-value.h>
-#include <libgnomeprint/gnome-font.h>
 
 #include "gedit-prefs-manager.h"
 #include "gedit-prefs-manager-private.h"
@@ -171,7 +170,7 @@ gedit_prefs_manager_init (void)
 			g_warning (_("Cannot initialize preferences manager."));
 			return FALSE;
 		}
-		
+
 		gedit_prefs_manager = g_new0 (GeditPrefsManager, 1);
 
 		gedit_prefs_manager->gconf_client = gconf_client;
@@ -188,7 +187,7 @@ gedit_prefs_manager_init (void)
 }
 
 void
-gedit_prefs_manager_shutdown ()
+gedit_prefs_manager_shutdown (void)
 {
 	gedit_debug (DEBUG_PREFS);
 
@@ -210,7 +209,7 @@ gedit_prefs_manager_get_bool (const gchar* key, gboolean def)
 						   key,
 						   def,
 						   NULL);
-}	
+}
 
 static gint 
 gedit_prefs_manager_get_int (const gchar* key, gint def)
@@ -294,93 +293,6 @@ gedit_prefs_manager_key_is_writable (const gchar* key)
 	return gconf_client_key_is_writable (gedit_prefs_manager->gconf_client, key, NULL);
 }
 
-static GdkColor
-gconf_client_get_color_with_default (GConfClient* client, const gchar* key,
-                      		     const gchar* def, GError** err)
-{
-	gchar *str_color = NULL;
-	GdkColor color = {0, };
-
-	gedit_debug (DEBUG_PREFS);
-
-      	g_return_val_if_fail (client != NULL, color);
-      	g_return_val_if_fail (GCONF_IS_CLIENT (client), color);  
-	g_return_val_if_fail (key != NULL, color);
-	g_return_val_if_fail (def != NULL, color);
-
-	str_color = gconf_client_get_string_with_default (client, 
-							  key, 
-							  def,
-							  NULL);
-
-	g_return_val_if_fail (str_color != NULL, color);
-
-	gdk_color_parse (str_color, &color);
-	g_free (str_color);
-	
-	return color;
-}
-
-static gboolean
-gedit_gconf_client_set_color (GConfClient   *client,
-                              const gchar   *key,
-                              GdkColor       val,
-                              GError       **err)
-{
-	gchar *str_color = NULL;
-	gboolean res;
-
-	gedit_debug (DEBUG_PREFS);
-
-	g_return_val_if_fail (client != NULL, FALSE);
-	g_return_val_if_fail (GCONF_IS_CLIENT (client), FALSE);  
-	g_return_val_if_fail (key != NULL, FALSE);
-
-	str_color = gedit_gdk_color_to_string (val);
-	g_return_val_if_fail (str_color != NULL, FALSE);
-
-	res = gconf_client_set_string (client,
-				       key,
-				       str_color,
-				       err);
-	g_free (str_color);
-
-	return res;
-}
-
-static GdkColor
-gedit_prefs_manager_get_color (const gchar* key, const gchar* def)
-{
-	GdkColor color = {0, };
-
-	gedit_debug (DEBUG_PREFS);
-
-	if (def != NULL)
-		gdk_color_parse (def, &color);
-
-	g_return_val_if_fail (gedit_prefs_manager != NULL, color);
-	g_return_val_if_fail (gedit_prefs_manager->gconf_client != NULL, color);
-
-	return gconf_client_get_color_with_default (gedit_prefs_manager->gconf_client,
-						    key,
-						    def,
-						    NULL);
-}	
-
-static void		 
-gedit_prefs_manager_set_color (const gchar* key, GdkColor value)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	g_return_if_fail (gedit_prefs_manager != NULL);
-	g_return_if_fail (gedit_prefs_manager->gconf_client != NULL);
-	g_return_if_fail (gconf_client_key_is_writable (
-				gedit_prefs_manager->gconf_client, key, NULL));
-			
-	gedit_gconf_client_set_color (gedit_prefs_manager->gconf_client, key, value, NULL);
-}
-
-
 /* Use default font */
 DEFINE_BOOL_PREF (use_default_font,
 		  GPM_USE_DEFAULT_FONT,
@@ -401,121 +313,6 @@ gedit_prefs_manager_get_system_font (void)
 					       GPM_DEFAULT_SYSTEM_FONT);
 }
 
-/* Use default colors */
-DEFINE_BOOL_PREF (use_default_colors,
-		  GPM_USE_DEFAULT_COLORS,
-		  GPM_DEFAULT_USE_DEFAULT_COLORS)
-
-
-/* Background color */	
-GdkColor
-gedit_prefs_manager_get_background_color (void)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	return gedit_prefs_manager_get_color (GPM_BACKGROUND_COLOR,
-					      GPM_DEFAULT_BACKGROUND_COLOR);
-}
-
-void
-gedit_prefs_manager_set_background_color (GdkColor color)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	gedit_prefs_manager_set_color (GPM_BACKGROUND_COLOR,
-				       color);
-}
-
-gboolean
-gedit_prefs_manager_background_color_can_set (void)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	return gedit_prefs_manager_key_is_writable (GPM_BACKGROUND_COLOR);
-}
-	
-/* Text color */	
-GdkColor
-gedit_prefs_manager_get_text_color (void)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	return gedit_prefs_manager_get_color (GPM_TEXT_COLOR,
-					      GPM_DEFAULT_TEXT_COLOR);
-}
-
-void
-gedit_prefs_manager_set_text_color (GdkColor color)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	gedit_prefs_manager_set_color (GPM_TEXT_COLOR,
-				       color);
-}
-
-gboolean
-gedit_prefs_manager_text_color_can_set (void)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	return gedit_prefs_manager_key_is_writable (GPM_TEXT_COLOR);
-}
-
-/* Selected text color */	
-GdkColor
-gedit_prefs_manager_get_selected_text_color (void)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	return gedit_prefs_manager_get_color (GPM_SELECTED_TEXT_COLOR,
-					      GPM_DEFAULT_SELECTED_TEXT_COLOR);
-}
-
-void
-gedit_prefs_manager_set_selected_text_color (GdkColor color)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	gedit_prefs_manager_set_color (GPM_SELECTED_TEXT_COLOR,
-				       color);
-}
-
-gboolean
-gedit_prefs_manager_selected_text_color_can_set (void)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	return gedit_prefs_manager_key_is_writable (GPM_SELECTED_TEXT_COLOR);
-}
-
-/* Selection color */	
-GdkColor
-gedit_prefs_manager_get_selection_color (void)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	return gedit_prefs_manager_get_color (GPM_SELECTION_COLOR,
-					      GPM_DEFAULT_SELECTION_COLOR);
-}
-
-void
-gedit_prefs_manager_set_selection_color (GdkColor color)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	gedit_prefs_manager_set_color (GPM_SELECTION_COLOR,
-				       color);
-}
-
-gboolean
-gedit_prefs_manager_selection_color_can_set (void)
-{
-	gedit_debug (DEBUG_PREFS);
-	
-	return gedit_prefs_manager_key_is_writable (GPM_SELECTION_COLOR);
-}
-
-
 /* Create backup copy */
 DEFINE_BOOL_PREF (create_backup_copy,
 		  GPM_CREATE_BACKUP_COPY,
@@ -772,8 +569,8 @@ gedit_prefs_manager_get_print_wrap_mode (void)
 void
 gedit_prefs_manager_set_print_wrap_mode (GtkWrapMode pwp)
 {
-	const gchar * str;
-	
+	const gchar *str;
+
 	gedit_debug (DEBUG_PREFS);
 
 	switch (pwp)
@@ -790,8 +587,7 @@ gedit_prefs_manager_set_print_wrap_mode (GtkWrapMode pwp)
 			str = "GTK_WRAP_CHAR";
 	}
 
-	gedit_prefs_manager_set_string (GPM_PRINT_WRAP_MODE,
-					str);
+	gedit_prefs_manager_set_string (GPM_PRINT_WRAP_MODE, str);
 }
 
 gboolean
@@ -806,213 +602,36 @@ gedit_prefs_manager_print_wrap_mode_can_set (void)
 DEFINE_INT_PREF (print_line_numbers,
 		 GPM_PRINT_LINE_NUMBERS,
 		 GPM_DEFAULT_PRINT_LINE_NUMBERS)
-		 
 
-/* The printing font entries are done in custom code because we
- * need to implement transitioning between old gnome-print font
- * names and new Pango fontnames
- */
-
-/*
- * The following routines are duplicated in gtksourceview/gtksourceview/gtksourceprintjob.c
- */
-
-/* Do this ourselves since gnome_font_find_closest() doesn't call
- * gnome_font_face_find_closest() (probably a gnome-print bug)
- */
-static void
-face_and_size_from_full_name (const gchar    *name,
-			      GnomeFontFace **face,
-			      gdouble        *size)
-{
-	char *copy;
-	char *str_size;
-
-	copy = g_strdup (name);
-	str_size = strrchr (copy, ' ');
-	if (str_size) {
-		*str_size = 0;
-		str_size ++;
-		*size = atof (str_size);
-	} else {
-		*size = 12;
-	}
-
-	*face = gnome_font_face_find_closest ((guchar *)copy);
-	g_free (copy);
-}
-
-static PangoFontDescription *
-font_description_from_gnome_font_name (const char *font_name)
-{
-	GnomeFontFace *face;
-	PangoFontDescription *desc;
-	PangoStyle style;
-	PangoWeight weight;
-	const gchar *family_name;
-	gdouble size;
-
-	face_and_size_from_full_name (font_name, &face, &size);
-
-	/* Pango and GnomePrint have basically the same numeric weight values */
-	weight = (PangoWeight) gnome_font_face_get_weight_code (face);
-	style = gnome_font_face_is_italic (face) ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL;
-	family_name = (const gchar *) gnome_font_face_get_family_name (face);
-
-	desc = pango_font_description_new ();
-	pango_font_description_set_family (desc, family_name);
-	pango_font_description_set_weight (desc, weight);
-	pango_font_description_set_style (desc, style);
-	pango_font_description_set_size (desc, size * PANGO_SCALE);
-
-	g_object_unref (face);
-
-	return desc;
-}
-
-static char *
-font_name_from_gnome_font_name (const char *gnome_name)
-{
-	PangoFontDescription *desc;
-	gchar *pango_name;
-
-	desc = font_description_from_gnome_font_name (gnome_name);
-	pango_name = pango_font_description_to_string (desc);
-	pango_font_description_free (desc);
-
-	return pango_name;
-}
-
-static gchar *
-get_string_without_default (GConfClient *gconf_client,
-			    const char  *key,
-			    GError     **error)
-{
-	GConfValue *value;
-	gchar *result = NULL;
-	
-	value = gconf_client_get_without_default (gconf_client, key, error);
-	if (value && value->type == GCONF_VALUE_STRING)
-	{
-		result = g_strdup (gconf_value_get_string (value));
-		gconf_value_free (value);
-	}
-
-	return result;
-}
-
-static gchar *
-gedit_prefs_manager_get_font (const gchar *pango_key,
-			      const gchar *gnome_print_key,
-			      const gchar *def)
-{
-	gchar *pango_value;
-	gchar *gnome_print_value;
-	
-	/* if the new pango key isn't writable, we don't want an old value to
-	 * overwrite it. Otherwise, we first look whether the pango key has
-	 * been set explicitely. If the pango key hasn't been set, but the
-	 * gnome-print key has, we use that and convert it into a pango font name.
-	 */
-	if (gedit_prefs_manager_key_is_writable (pango_key))
-	{
-		pango_value = get_string_without_default (gedit_prefs_manager->gconf_client,
-							  pango_key,
-							  NULL);
-		if (pango_value)
-			return pango_value;
-		
-		gnome_print_value = get_string_without_default (gedit_prefs_manager->gconf_client,
-								gnome_print_key,
-								NULL);
-
-		if (gnome_print_value) {
-			pango_value = font_name_from_gnome_font_name (gnome_print_value);
-			g_free (gnome_print_value);
-			
-			return pango_value;
-		}
-	}
-
-	return gedit_prefs_manager_get_string (pango_key, def);
-}
-
-gchar *
-gedit_prefs_manager_get_print_font_body (void)
-{
-	return gedit_prefs_manager_get_font (GPM_PRINT_FONT_BODY_PANGO,
-					     GPM_PRINT_FONT_BODY,
-					     GPM_DEFAULT_PRINT_FONT_BODY_PANGO);
-}
-
-void
-gedit_prefs_manager_set_print_font_body (const gchar* v)
-{
-	gedit_prefs_manager_set_string (GPM_PRINT_FONT_BODY_PANGO, v);
-}
-
-gboolean
-gedit_prefs_manager_print_font_body_can_set (void)
-{
-	return gedit_prefs_manager_key_is_writable (GPM_PRINT_FONT_BODY_PANGO);
-}		
+/* Printing fonts */
+DEFINE_STRING_PREF (print_font_body,
+		    GPM_PRINT_FONT_BODY,
+		    GPM_DEFAULT_PRINT_FONT_BODY)
 
 const gchar *
 gedit_prefs_manager_get_default_print_font_body (void)
 {
-	return GPM_DEFAULT_PRINT_FONT_BODY_PANGO;
+	return GPM_DEFAULT_PRINT_FONT_BODY;
 }
 
-gchar *
-gedit_prefs_manager_get_print_font_header (void)
-{
-	return gedit_prefs_manager_get_font (GPM_PRINT_FONT_HEADER_PANGO,
-					     GPM_PRINT_FONT_HEADER,
-					     GPM_DEFAULT_PRINT_FONT_HEADER_PANGO);
-}
-
-void
-gedit_prefs_manager_set_print_font_header (const gchar* v)
-{
-	gedit_prefs_manager_set_string (GPM_PRINT_FONT_HEADER_PANGO, v);
-}
-
-gboolean
-gedit_prefs_manager_print_font_header_can_set (void)
-{
-	return gedit_prefs_manager_key_is_writable (GPM_PRINT_FONT_HEADER_PANGO);
-}		
+DEFINE_STRING_PREF (print_font_header,
+		    GPM_PRINT_FONT_HEADER,
+		    GPM_DEFAULT_PRINT_FONT_HEADER)
 
 const gchar *
 gedit_prefs_manager_get_default_print_font_header (void)
 {
-	return GPM_DEFAULT_PRINT_FONT_HEADER_PANGO;
+	return GPM_DEFAULT_PRINT_FONT_HEADER;
 }
 
-gchar *
-gedit_prefs_manager_get_print_font_numbers (void)
-{
-	return gedit_prefs_manager_get_font (GPM_PRINT_FONT_NUMBERS_PANGO,
-					     GPM_PRINT_FONT_NUMBERS,
-					     GPM_DEFAULT_PRINT_FONT_NUMBERS_PANGO);
-}
-
-void
-gedit_prefs_manager_set_print_font_numbers (const gchar* v)
-{
-	gedit_prefs_manager_set_string (GPM_PRINT_FONT_NUMBERS_PANGO, v);
-}
-
-gboolean
-gedit_prefs_manager_print_font_numbers_can_set (void)
-{
-	return gedit_prefs_manager_key_is_writable (GPM_PRINT_FONT_NUMBERS_PANGO);
-}		
+DEFINE_STRING_PREF (print_font_numbers,
+		    GPM_PRINT_FONT_NUMBERS,
+		    GPM_DEFAULT_PRINT_FONT_NUMBERS)
 
 const gchar *
 gedit_prefs_manager_get_default_print_font_numbers (void)
 {
-	return GPM_DEFAULT_PRINT_FONT_NUMBERS_PANGO;
+	return GPM_DEFAULT_PRINT_FONT_NUMBERS;
 }
 
 /* Max number of files in "Recent Files" menu. 
@@ -1050,6 +669,7 @@ gedit_prefs_manager_get_auto_detected_encodings (void)
 {
 	GSList *strings;
 	GSList *res = NULL;
+
 	gedit_debug (DEBUG_PREFS);
 
 	g_return_val_if_fail (gedit_prefs_manager != NULL, NULL);
@@ -1119,6 +739,7 @@ gedit_prefs_manager_get_shown_in_menu_encodings (void)
 {
 	GSList *strings;
 	GSList *res = NULL;
+
 	gedit_debug (DEBUG_PREFS);
 
 	g_return_val_if_fail (gedit_prefs_manager != NULL, NULL);
@@ -1194,8 +815,8 @@ gedit_prefs_manager_set_shown_in_menu_encodings (const GSList *encs)
 			GPM_SHOWN_IN_MENU_ENCODINGS,
 			GCONF_VALUE_STRING,
 		       	list,
-			NULL);	
-	
+			NULL);
+
 	g_slist_free (list);
 }
 
@@ -1228,6 +849,79 @@ DEFINE_INT_PREF (right_margin_position,
 		 GPM_RIGHT_MARGIN_POSITION,
 		 GPM_DEFAULT_RIGHT_MARGIN_POSITION)
 
+static GtkSourceSmartHomeEndType
+get_smart_home_end_from_string (const gchar *str)
+{
+	GtkSourceSmartHomeEndType res;
+
+	g_return_val_if_fail (str != NULL, GTK_SOURCE_SMART_HOME_END_AFTER);
+
+	if (strcmp (str, "DISABLED") == 0)
+		res = GTK_SOURCE_SMART_HOME_END_DISABLED;
+	else if (strcmp (str, "BEFORE") == 0)
+		res = GTK_SOURCE_SMART_HOME_END_BEFORE;
+	else if (strcmp (str, "ALWAYS") == 0)
+		res = GTK_SOURCE_SMART_HOME_END_ALWAYS;
+	else
+		res = GTK_SOURCE_SMART_HOME_END_AFTER;
+
+	return res;
+}
+
+GtkSourceSmartHomeEndType
+gedit_prefs_manager_get_smart_home_end (void)
+{
+	gchar *str;
+	GtkSourceSmartHomeEndType res;
+
+	gedit_debug (DEBUG_PREFS);
+
+	str = gedit_prefs_manager_get_string (GPM_SMART_HOME_END,
+					      GPM_DEFAULT_SMART_HOME_END);
+
+	res = get_smart_home_end_from_string (str);
+
+	g_free (str);
+
+	return res;
+}
+	
+void
+gedit_prefs_manager_set_smart_home_end (GtkSourceSmartHomeEndType smart_he)
+{
+	const gchar *str;
+
+	gedit_debug (DEBUG_PREFS);
+
+	switch (smart_he)
+	{
+		case GTK_SOURCE_SMART_HOME_END_DISABLED:
+			str = "DISABLED";
+			break;
+
+		case GTK_SOURCE_SMART_HOME_END_BEFORE:
+			str = "BEFORE";
+			break;
+
+		case GTK_SOURCE_SMART_HOME_END_ALWAYS:
+			str = "ALWAYS";
+			break;
+
+		default: /* GTK_SOURCE_SMART_HOME_END_AFTER */
+			str = "AFTER";
+	}
+
+	gedit_prefs_manager_set_string (GPM_WRAP_MODE, str);
+}
+
+gboolean
+gedit_prefs_manager_smart_home_end_can_set (void)
+{
+	gedit_debug (DEBUG_PREFS);
+	
+	return gedit_prefs_manager_key_is_writable (GPM_SMART_HOME_END);
+}
+
 /* Enable syntax highlighting */
 DEFINE_BOOL_PREF (enable_syntax_highlighting,
 		  GPM_SYNTAX_HL_ENABLE,
@@ -1238,6 +932,10 @@ DEFINE_BOOL_PREF (enable_search_highlighting,
 		  GPM_SEARCH_HIGHLIGHTING_ENABLE,
 		  GPM_DEFAULT_SEARCH_HIGHLIGHTING_ENABLE)
 
+/* Source style scheme */
+DEFINE_STRING_PREF (source_style_scheme,
+		    GPM_SOURCE_STYLE_SCHEME,
+		    GPM_DEFAULT_SOURCE_STYLE_SCHEME)
 
 GSList *
 gedit_prefs_manager_get_writable_vfs_schemes (void)
@@ -1286,7 +984,51 @@ gedit_prefs_manager_get_restore_cursor_position (void)
 					     GPM_DEFAULT_RESTORE_CURSOR_POSITION);
 }
 
+/* Plugins: we just store/return a list of strings, all the magic has to
+ * happen in the plugin engine */
+
+GSList *
+gedit_prefs_manager_get_active_plugins (void)
+{
+	GSList *plugins;
+
+	gedit_debug (DEBUG_PREFS);
+
+	g_return_val_if_fail (gedit_prefs_manager != NULL, NULL);
+	g_return_val_if_fail (gedit_prefs_manager->gconf_client != NULL, NULL);
+
+	plugins = gconf_client_get_list (gedit_prefs_manager->gconf_client,
+					 GPM_ACTIVE_PLUGINS,
+					 GCONF_VALUE_STRING, 
+					 NULL);
+
+	return plugins;
+}
+
+void
+gedit_prefs_manager_set_active_plugins (const GSList *plugins)
+{	
+	g_return_if_fail (gedit_prefs_manager != NULL);
+	g_return_if_fail (gedit_prefs_manager->gconf_client != NULL);
+	g_return_if_fail (gedit_prefs_manager_active_plugins_can_set ());
+
+	gconf_client_set_list (gedit_prefs_manager->gconf_client,
+			       GPM_ACTIVE_PLUGINS,
+			       GCONF_VALUE_STRING,
+		       	       (GSList *) plugins,
+			       NULL);
+}
+
+gboolean
+gedit_prefs_manager_active_plugins_can_set (void)
+{
+	gedit_debug (DEBUG_PREFS);
+
+	return gedit_prefs_manager_key_is_writable (GPM_ACTIVE_PLUGINS);
+}
+
 /* Global Lockdown */
+
 GeditLockdownMask
 gedit_prefs_manager_get_lockdown (void)
 {
