@@ -50,7 +50,6 @@
 #include "gedit-debug.h"
 #include "gedit-dirs.h"
 #include "gedit-encodings.h"
-#include "gedit-metadata-manager.h"
 #include "gedit-plugins-engine.h"
 #include "gedit-prefs-manager-app.h"
 #include "gedit-session.h"
@@ -67,10 +66,16 @@
 #include <conio.h>
 #define _WIN32_WINNT 0x0500
 #include <windows.h>
+#include "gedit-metadata-manager.h"
 #define DATADIR SAVE_DATADIR
 #undef SAVE_DATADIR
 #endif
 
+#ifdef OS_OSX
+#include <ige-mac-dock.h>
+#include "osx/gedit-osx.h"
+#endif
+
 static guint32 startup_timestamp = 0;
 
 #ifndef G_OS_WIN32
@@ -82,7 +87,6 @@ static BaconMessageConnection *connection;
 /* command line */
 static gint line_position = 0;
 static gchar *encoding_charset = NULL;
-static const GeditEncoding *encoding;
 static gboolean new_window_option = FALSE;
 static gboolean new_document_option = FALSE;
 static gchar **remaining_args = NULL;
@@ -135,6 +139,24 @@ static const GOptionEntry options [] =
 	{NULL}
 };
 
+static void
+free_command_line_data (void)
+{
+	g_slist_foreach (file_list, (GFunc) g_object_unref, NULL);
+	g_slist_free (file_list);
+	file_list = NULL;
+
+	g_strfreev (remaining_args);
+	remaining_args = NULL;
+
+	g_free (encoding_charset);
+	encoding_charset = NULL;
+	
+	new_window_option = FALSE;
+	new_document_option = FALSE;
+	line_position = 0;
+}
+
 static void
 gedit_get_command_line_data (void)
 {
@@ -163,21 +185,17 @@ gedit_get_command_line_data (void)
 
 		file_list = g_slist_reverse (file_list);
 	}
-
-	if (encoding_charset)
+	
+	if (encoding_charset &&
+	    (gedit_encoding_get_from_charset (encoding_charset) == NULL))
 	{
-		encoding = gedit_encoding_get_from_charset (encoding_charset);
-		if (encoding == NULL)
-			g_print (_("%s: invalid encoding.\n"),
-				 encoding_charset);
-
-		g_free (encoding_charset);
-		encoding_charset = NULL;
+		g_print (_("%s: invalid encoding.\n"),
+			 encoding_charset);
 	}
 }
 
 static guint32
-get_startup_timestamp ()
+get_startup_timestamp (void)
 {
 	const gchar *startup_id_env;
 	gchar *startup_id = NULL;
@@ -241,6 +259,7 @@ static void
 on_message_received (const char *message,
 		     gpointer    data)
 {
+	const GeditEncoding *encoding = NULL;
 	gchar **commands;
 	gchar **params;
 	gint workspace;
@@ -300,7 +319,8 @@ on_message_received (const char *message,
 			gchar **uris;
 
 			line_position = atoi (params[1]);
-			if (params[2] != '\0');
+
+			if (params[2] != '\0')
 				encoding = gedit_encoding_get_from_charset (params[2]);
 
 			n_uris = atoi (params[3]);
@@ -378,26 +398,18 @@ on_message_received (const char *message,
 
 #ifdef GDK_WINDOWING_X11
 	if (startup_timestamp <= 0)
-		startup_timestamp = gdk_x11_get_server_time (GTK_WIDGET (window)->window);
+		startup_timestamp = gdk_x11_get_server_time (gtk_widget_get_window (GTK_WIDGET (window)));
 
-	gdk_x11_window_set_user_time (GTK_WIDGET (window)->window,
+	gdk_x11_window_set_user_time (gtk_widget_get_window (GTK_WIDGET (window)),
 				      startup_timestamp);
 #endif
 
 	gtk_window_present (GTK_WINDOW (window));
 
-	/* free the file list and reset to default */
-	g_slist_foreach (file_list, (GFunc) g_object_unref, NULL);
-	g_slist_free (file_list);
-	file_list = NULL;
-
  out:
 	g_strfreev (commands);
-
-	new_window_option = FALSE;
-	new_document_option = FALSE;
-	encoding = NULL;
-	line_position = 0;
+	
+	free_command_line_data ();
 }
 
 /* clientside */
@@ -541,6 +553,9 @@ main (int argc, char *argv[])
 	gchar *dir;
 	gchar *icon_dir;
 
+	/* Init type system as soon as possible */
+	g_type_init ();
+
 	/* Init glib threads asap */
 	g_thread_init (NULL);
 
@@ -617,7 +632,9 @@ main (int argc, char *argv[])
 			gedit_get_command_line_data ();
 
 			send_bacon_message ();
-
+			
+			free_command_line_data ();
+			
 			/* we never popup a window... tell startup-notification
 			 * that we are done.
 			 */
@@ -654,7 +671,7 @@ main (int argc, char *argv[])
 					   icon_dir);
 	g_free (icon_dir);
 
-#if !defined PLATFORM_OSX && !defined G_OS_WIN32
+#ifdef GDK_WINDOWING_X11
 	/* Set the associated .desktop file */
 	egg_set_desktop_file (DATADIR "/applications/gedit.desktop");
 #else
@@ -693,6 +710,11 @@ main (int argc, char *argv[])
 
 		if (file_list != NULL)
 		{
+			const GeditEncoding *encoding = NULL;
+		
+			if (encoding_charset)
+				encoding = gedit_encoding_get_from_charset (encoding_charset);
+		
 			gedit_debug_message (DEBUG_APP, "Load files");
 			_gedit_cmd_load_files_from_prompt (window, 
 							   file_list, 
@@ -708,19 +730,14 @@ main (int argc, char *argv[])
 		gedit_debug_message (DEBUG_APP, "Show window");
 		gtk_widget_show (GTK_WIDGET (window));
 
-		g_slist_foreach (file_list, (GFunc) g_object_unref, NULL);
-		g_slist_free (file_list);
-		file_list = NULL;
-
-		g_free (encoding_charset);
-
-		new_window_option = FALSE;
-		new_document_option = FALSE;
-		encoding = NULL;
-		line_position = 0;
+		free_command_line_data ();
 	}
 
-	gedit_debug_message (DEBUG_APP, "Start gtk-main");		
+	gedit_debug_message (DEBUG_APP, "Start gtk-main");
+	
+#ifdef OS_OSX
+	gedit_osx_init(gedit_app_get_default ());
+#endif
 	gtk_main();
 
 #ifndef G_OS_WIN32
@@ -732,7 +749,10 @@ main (int argc, char *argv[])
 	 */
 	g_object_unref (engine);
 	gedit_prefs_manager_app_shutdown ();
+
+#ifdef G_OS_WIN32
 	gedit_metadata_manager_shutdown ();
+#endif
 
 	return 0;
 }
