@@ -32,7 +32,6 @@
 #include <gmodule.h>
 
 #include <gedit/gedit-debug.h>
-#include <gedit/gedit-metadata-manager.h>
 #include <gedit/gedit-prefs-manager.h>
 #include <gedit/gedit-statusbar.h>
 
@@ -41,6 +40,13 @@
 #include "gedit-spell-language-dialog.h"
 #include "gedit-automatic-spell-checker.h"
 
+#ifdef G_OS_WIN32
+#include <gedit/gedit-metadata-manager.h>
+#define GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE "spell-language"
+#else
+#define GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE "metadata::gedit-spell-language"
+#endif
+
 #define WINDOW_DATA_KEY "GeditSpellPluginWindowData"
 #define MENU_PATH "/MenuBar/ToolsMenu/ToolsOps_1"
 
@@ -134,26 +140,16 @@ set_spell_language_cb (GeditSpellChecker   *spell,
 		       const GeditSpellCheckerLanguage *lang,
 		       GeditDocument 	   *doc)
 {
-	gchar *uri;
+	const gchar *key;
 
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 	g_return_if_fail (lang != NULL);
 
-	uri = gedit_document_get_uri (doc);
+	key = gedit_spell_checker_language_to_key (lang);
+	g_return_if_fail (key != NULL);
 
-	if (uri != NULL)
-	{
-		const gchar *key;
-
-		key = gedit_spell_checker_language_to_key (lang);
-		g_return_if_fail (key != NULL);
-
-		gedit_metadata_manager_set (uri, 
-					    "spell-language",
-					    key);
-
-		g_free (uri);
-	}
+	gedit_document_set_metadata (doc, GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE,
+				     key, NULL);
 }
 
 static GeditSpellChecker *
@@ -170,31 +166,22 @@ get_spell_checker_from_document (GeditDocument *doc)
 
 	if (data == NULL)
 	{
-		gchar *uri;
+		const GeditSpellCheckerLanguage *lang = NULL;
+		gchar *value = NULL;
 
 		spell = gedit_spell_checker_new ();
 
-		uri = gedit_document_get_uri (doc);
+		value = gedit_document_get_metadata (doc, GEDIT_METADATA_ATTRIBUTE_SPELL_LANGUAGE);
 
-		if (uri != NULL)
+		if (value != NULL)
 		{
-			gchar *key;
-			const GeditSpellCheckerLanguage *lang = NULL;
+			lang = gedit_spell_checker_language_from_key (value);
+			g_free (value);
+		}
 
-			key = gedit_metadata_manager_get (uri,
-							  "spell-language");
-
-			if (key != NULL)
-			{
-				lang = gedit_spell_checker_language_from_key (key);
-				g_free (key);
-			}
-
-			if (lang != NULL)
-				gedit_spell_checker_set_language (spell,
-								  lang);
-
-			g_free (uri);	
+		if (lang != NULL)
+		{
+			gedit_spell_checker_set_language (spell, lang);
 		}
 
 		g_object_set_qdata_full (G_OBJECT (doc), 
